library IEEE;
   use IEEE.std_logic_1164.all;
   USE ieee.std_logic_arith.all ;
   USE ieee.std_logic_unsigned.all ;

entity dsmc_demo_FSM is
       port (  
--	//////////// CLOCK //////////
	CLOCK_50 		: IN STD_LOGIC;

--	//////////// LED //////////
	LED			:OUT STD_LOGIC_VECTOR(7 downto 0);

--	//////////// GPIO_0, GPIO_0 connect to GPIO Default //////////
	GPIO_0			:INOUT STD_LOGIC_VECTOR(33 downto 0);
	GPIO_0_IN 		:IN STD_LOGIC_VECTOR(1 downto 0);
--	//////////// GPIO_1, GPIO_1 connect to GPIO Default //////////
	GPIO_1			:INOUT STD_LOGIC_VECTOR(33 downto 0);
	GPIO_1_IN		:IN STD_LOGIC_VECTOR(1 downto 0)
                             );
end dsmc_demo_FSM ;

architecture rtl of dsmc_demo_FSM is
signal led_s : STD_LOGIC_VECTOR(7 downto 0);
signal clk, rst, inp, inp_d, inp_2d : STD_LOGIC;
signal cnt1, cnt2 : STD_LOGIC_VECTOR( 19 downto 0);

TYPE state_type IS (IDLE,DELAY, LED_ON);
signal state_cur, state_nxt: state_type;

begin
clk <= CLOCK_50;
inp <= GPIO_0(0);
rst <= GPIO_0(13);

process (clk, rst)
begin
	if rst='1' then
			cnt1<=(others=>'0');
			cnt2<=(others=>'0');
			inp_d<='0';		-- delay 1xCLK
			inp_2d<='0';		-- delay 1xCLK
			state_cur<=IDLE;
 	elsif (clk'event and clk = '1') then
				inp_d<=inp;		-- delay 1xCLK
				inp_2d<=inp_d;		-- delay 1xCLK
				state_cur<=state_nxt;
				if (state_cur=DELAY) then
					cnt1<=cnt1+1;
				else 
					cnt1<=(others=>'0');
					cnt2<=(others=>'0');
				end if;
				if (cnt1=1000000) then
					cnt2<=cnt2+1;
					cnt1<=(others=>'0');
				end if;
				if (cnt2=300) then
					cnt2<=(others=>'0');
				end if;
	end if;
end process;

fsm: process (state_cur, inp, inp_d, cnt2)
begin
	case state_cur is
		when IDLE =>
			if (inp_d='1') AND (inp_2d='0') then
				state_nxt<=DELAY;
			else	
				state_nxt<=IDLE;
			end if;
		when DELAY =>
			if (cnt2=300) then
				state_nxt<=LED_ON;	-- Wait for 300x20msec=6 sec
			else
				state_nxt<=DELAY;
			end if;
		when LED_ON =>
				state_nxt<=LED_ON; 	-- Stay ON until reset
	end case;
end process;

with state_cur select led_s <= (others=>'1') when LED_ON, (others=>'0') when others;
LED <= led_s;
end rtl ;
